;=======================================================
; Para-Dry Bones
; By Erik557
;
; Description: A dry bones that gives jumps similar
; to the green bouncing parakoopa (sprite 09).
;
; Uses first extra bit: YES
; Extra bit unset: Y position determines height of the
; jumps.
; Extra bit set: Alternate between high and low jumps.
;
; Uses first extra property byte: YES
; 00 - Spawn sprite 32 when hit.
; 01 - Spawn sprite 30 when hit.
;=======================================================

;======================
; Defines and stuff
;======================

DryBonesTiles:
       db $64,$66,$64,$68
WingsTiles:
       db $5D,$C6,$5D,$C6
WingProps:
       db $76,$76,$36,$36
XSpeeds:
       db $08,$F8
YSpeeds:
       db $D0,$B0

;=================================
; INIT and MAIN Wrappers
;=================================

print "INIT ",pc
       JSL $01ACF9
       STA $1570,x
       LDA $7FAB10,x
       AND #$04
       BNE +
       LDA $D8,x
       AND #$10
       LSR #4
       BRA .join
+      LDA $148D
       AND #$01
.join
       STA $C2,x
       JSR SubHorizPos
       TYA
       STA $157C,x
       TAX
       LDA.l XSpeeds,x
       LDX $15E9
       STA $B6,x
       RTL

print "MAIN ",pc
       PHB
       PHK
       PLB
       JSR paraBones
       PLB
       RTL

paraBones_return:
       RTS
paraBones:
       JSR Graphics
       LDA $14C8,x
       EOR #$08
       ORA $9D
       BNE .return
       LDY #$00
       JSR SubOffScreen
       JSL $01802A
       JSL $018032

       INC $1570,x
       LDA $1570,x
       LSR #3
       AND #$01
       STA $1602,x

       LDY $157C,x
       LDA XSpeeds,y
       CMP $B6,x
       BEQ +
       STA $B6,x
+      LDA $157C,x
       INC
       AND $1588,x
       AND #$03
       BEQ .SetY
       LDA $15AC,x
       BNE .SetY
       LDA #$08
       STA $15AC,x
       LDA $B6,x
       EOR #$FF
       INC
       STA $B6,x
       LDA $157C,x
       EOR #$01
       STA $157C,x

.SetY
       DEC $AA,x
       LDA $1588,x          ;\
       BIT #$08             ; |
       BEQ +                ; | Reset speed if touching ceiling
       STZ $AA,x            ; |
       BRA +                ;/
+      AND #$04             ;\ Don't update while on air
       BEQ +                ;/
       LDA $C2,x            ;\
       AND #$01             ; |
       TAY                  ; | Increase speed
       LDA YSpeeds,y        ; |
       STA $AA,x            ;/
       LDA $7FAB10,x
       AND #$04
       BEQ +
       INC $C2,x

+      JSL $01A7DC
       BCC +
       LDA $D3
       CLC
       ADC #$14
       CMP $D8,x
       BMI .cont
.hurt
       LDA $1497
       BNE +
       JSL $00F5B7
       LDX $15E9
       JSR SubHorizPos
       TYA
       STA $157C,x
+      RTS
.cont
       LDA $7D
       BMI .hurt
       JSL $01AB99
       JSL $01AA33

       LDA $7FAB28,x
       TAY
       LDA SprToSpawn,y
       STA $9E,x

       LDA $1626,x
       PHA
       JSL $07F7D2
       PLA
       STA $1626,x

       JSR SubHorizPos
       TYA
       STA $157C,x
       INC $1534,x
       LDA #$FF
       STA $1540,x

       PHY
       LDA $1697
       CLC
       ADC $1626,x
       INC $1697
       CMP #$08
       BCC +
       LDA #$08
+      JSL $02ACE5
       LDA #$07
       STA $1DF9
       PLY
       RTS

SprToSpawn:
       db $32,$30
KillSFX:
       db $00,$13,$14,$15,$16,$17,$18,$19

;========================
; Graphics routine
;========================

Graphics:
       JSR GetDrawInfo
       LDA $1602,x          ;\  tile index for future uses
       STA $02              ;/
       LDA $157C,x          ;\
       ASL                  ; | multiply the direction
       CLC                  ; | add our frame index to index the tables
       ADC $02              ; |
       TAX                  ;/
       LDA $00              ;\
       CLC                  ; | wings x pos
       ADC WingsXDisp,x     ; |
       STA $0300,y          ;/
       LDA $01              ;\
       CLC                  ; | wings y pos
       ADC WingsYDisp,x     ; |
       STA $0301,y          ;/
       LDA WingsTiles,x     ;\  wings tilemap
       STA $0302,y          ;/
       LDA $64              ;\
       ORA WingProps,x      ; | wings properties
       STA $0303,y          ;/
       TYA                  ;\
       LSR #2               ; | index into tilesize
       TAY                  ;/
       LDA WingsSize,x      ;\  variable size, depending on the wing frame
       STA $0460,y          ;/

       LDX $15E9            ;   retrieve index
       LDA $15EA,x          ;\
       CLC                  ; | get new set of oam slots
       ADC #$04             ; |
       TAY                  ;/

       LDA $02
       ASL
       STA $02
       LDA $157C,x
       ASL #2
       STA $03

       LDX #$01
-      PHX
       TXA
       CLC
       ADC $02
       TAX
       LDA $01              ;\
       CLC                  ; | y pos
       ADC YDisp,x          ; |
       STA $0301,y          ;/

       PHX
       LDA DryBonesTiles,x  ;\  tilemap
       STA $0302,y          ;/
       LDX $15E9            ;\
       LDA $157C,x          ; |
       TAX                  ; | properties
       LDA Props,x          ; |
       ORA #$23             ; |
       STA $0303,y          ;/

       PLA
       CLC
       ADC $03
       TAX
       LDA $00              ;\
       CLC                  ; | x pos
       ADC XDisp,x          ; |
       STA $0300,y          ;/

       TYA                  ;\
       LSR #2               ; | index into tilesize
       TAY                  ;/
       LDA #$02             ;\  spiny is 16x16
       STA $0460,y          ;/

       LDX $15E9
       LDA $15EA,x
       CLC
       ADC #$08
       TAY

       PLX
       DEX
       BPL -

       LDX $15E9
       LDY #$FF             ;   variable size
       LDA #$02             ;   3 tiles
       JSL $01B7B3          ;   finish write
       RTS

WingsSize:
       db $00,$02,$00,$02
WingsXDisp:
       db $FF,$F8,$08,$0A
WingsYDisp:
       db $02,$FA,$02,$FA
XDisp:
       db $08,$00,$08,$00
       db $F8,$00,$F8,$00
YDisp:
       db $F0,$00,$F0,$00
Props:
       db $40,$00

;========================
; Misc. subroutines
;========================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  GetDrawInfo:            ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$20
       SEC
       SBC $1A
       STA $00
       CLC
       ADC #$0040
       CMP #$0180
       SEP #$20
       LDA $01
       BEQ +
       LDA #$01
+      STA $15A0,x
       TDC
       ROL A
       STA $15C4,x
       BNE .Invalid
       LDA $14D4,x
       XBA
       LDA $1662,x
       AND #$20
       BEQ .CheckOnce
.CheckTwice
       LDA $D8,x
       REP #$21
       ADC #$001C
       SEC
       SBC $1C
       SEP #$20
       LDA $14D4,x
       XBA
       BEQ .CheckOnce
       LDA #$02
.CheckOnce
       STA $186C,x
       LDA $D8,x
       REP #$21
       ADC #$000C
       SEC
       SBC $1C
       SEP #$21
       SBC #$0C
       STA $01
       XBA
       BEQ .OnScreenY
       INC $186C,x
.OnScreenY
       LDY $15EA,x
       RTS

.Invalid
       PLA
       PLA
       RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  SubOffScreen:           ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       
       LDA $15A0,x
       ORA $186C,x
       BEQ .Return2
       LDA $167A,x
       AND #$04
       BNE .Return2
       LDA $5B
       LSR A
       BCS .VerticalLevel
       LDA $D8,x
       ADC #$50
       LDA $14D4,x
       ADC #$00
       CMP #$02
       BCS .EraseSprite
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC .AddTable,y
       SEC
       SBC $1A
       CMP .CmpTable,y
.Common
       SEP #$20
       BCC .Return2
.EraseSprite
       LDA $14C8,x
       CMP #$08
       BCC .KillSprite
       LDY $161A,x
       CPY #$FF
       BEQ .KillSprite
       LDA $1938,y
       AND #$FE
       STA $1938,y
.KillSprite
       STZ $14C8,x
.Return2
       RTS

.VerticalLevel
       LDA $13
       LSR A
       BCS .CheckY
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC.w #$0040
       CMP.w #$0280
       BRA .Common
.CheckY
       LDA $14D4,x
       XBA
       LDA $D8,x
       REP #$20
       SBC $1C
       CLC
       ADC.w #$0070
       CMP.w #$01D0
       BRA .Common

.AddTable
       dw $0040,$0040,$0010,$0070
       dw $0090,$0050,$0080,$FFC0
.CmpTable
       dw $0170,$01E0,$01B0,$01D0
       dw $0230,$01B0,$0220,$0160

;==================
  SubHorizPos:
;==================

       LDY #$00
       LDA $94
       SEC
       SBC $E4,x
       STA $0F
       LDA $95
       SBC $14E0,x
       BPL $01
       INY
       RTS

